/*
 * Copyright 2017 S. Koulouzis, Wang Junchao, Huan Zhou, Yang Hu 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.uva.sne.drip.drip.commons.data.v1.external;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.webcohesion.enunciate.metadata.DocumentationExample;
import java.util.Set;
import org.springframework.data.mongodb.core.mapping.Document;

/**
 *
 * This class represents a plan generated by the planner.
 *
 * @author S. Koulouzis
 */
@Document
@JsonInclude(JsonInclude.Include.NON_NULL)
public class PlanResponse extends KeyValueHolder {

    private String toscaID;

    private String name;

    private Integer level;

    private Set<String> loweLevelPlansIDs;

    /**
     * The name of the generated plan
     *
     * @return the name
     */
    @DocumentationExample("planner_output_all.yml")
    public String getName() {
        return name;
    }

    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * The id of the TOSCA <code>PlaybookRepresentation</code> description from
     * which this plan was generated
     *
     * @return the toscaID
     */
    @DocumentationExample("58e3946e0fb4f562d84ba1ad")
    public String getToscaID() {
        return toscaID;
    }

    /**
     * @param toscaID the toscaID to set
     */
    public void setToscaID(String toscaID) {
        this.toscaID = toscaID;
    }

    /**
     * The level of the plan. Some provisioners use levels to represent a plan.
     * In these cases there are two levels of TOSCA descriptions.
     *
     * @return the level
     */
    @DocumentationExample("0")
    public Integer getLevel() {
        return level;
    }

    /**
     * @param level the level to set
     */
    public void setLevel(Integer level) {
        this.level = level;
    }

    /**
     * The list of the lower level plans attached to this plan. Some
     * provisioners use levels to represent a plan. In these cases there are two
     * levels of TOSCA descriptions.
     *
     *
     * @return the loweLevelPlansIDs
     */
    @DocumentationExample("ew44Ae3946e0sdds4f562d84baA")
    public Set<String> getLoweLevelPlanIDs() {
        return loweLevelPlansIDs;
    }

    public void setLoweLevelPlansIDs(Set<String> loweLevelPlansIDs) {
        this.loweLevelPlansIDs = loweLevelPlansIDs;
    }
}
